/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __CONTROL_H
#define __CONTROL_H

#include <QObject>
#include <QList>
#include <MS_Dl.h>
#include <MS_Virtusers.h>

typedef void (*cmdParam)(MUser *, const QStringList &);
typedef struct cmdHandler {
  QString command;
  QString description;
  QString helpmsg;
  cmdParam handler;
} cmdHandler;

class ModControl : public QObject, MModInterface {
  Q_OBJECT
  Q_INTERFACES(MModInterface)
  
  private:
    bool is_nickCreated;
    MModule *m_modptr;
    MVirtualUser *controluser;
    QString numericstr;
    QList<cmdHandler *> handlers;

  public:
    ModControl();
    ~ModControl();
    void init(void);
    void registerCommand(const QString &strCommand, const QString &strDesc, const QString &strHelp, cmdParam handler);
    void deregisterCommand(const QString &strCommand);
    void sendnotice(MUser *user, const QString &strMessage);
    const QList<cmdHandler *> &getHandlers(void);

    /* commands */
    static void c_get(MUser *sender, const QStringList &strlArgs);
    static void c_help(MUser *sender, const QStringList &strlArgs);
    static void c_insmod(MUser *sender, const QStringList &strlArgs);
    static void c_lsmod(MUser *sender, const QStringList &strlArgs);
    static void c_showcommands(MUser *sender, const QStringList &strlArgs);
    static void c_rlmod(MUser *sender, const QStringList &strlArgs);
    static void c_rmmod(MUser *sender, const QStringList &strlArgs);
    static void c_serverlist(MUser *sender, const QStringList &strlArgs);
    static void c_set(MUser *sender, const QStringList &strlArgs);
    static void c_showchan(MUser *sender, const QStringList &strlArgs);
    static void c_stats(MUser *sender, const QStringList &strlArgs);
    static void c_status(MUser *sender, const QStringList &strlArgs);
    static void c_whois(MUser *sender, const QStringList &strlArgs);

  public slots:
    void sendNickCreation(const QStringList &strlArgs);
    void handlePrivmsg(const QStringList &strlArgs);
};

extern ModControl *__ModControl;

#endif /* __CONTROL_H */
