/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MS_P10_PARSER_H
#define __MS_P10_PARSER_H

#include <QString>
#include <QStringList>
#include <QList>
#include "MS_Flagops.h"

/* base64 encoder and decoder used from ircu2.10
 * (C) 2002 Undernet coder com
 */
#define NUMNICKLOG 6
#define NUMNICKMAXCHAR 'z'      /* See convert2n[] */
#define NUMNICKBASE 64          /* (2 << NUMNICKLOG) */
#define NUMNICKMASK 63          /* (NUMNICKBASE-1) */

class MP10Parser : public QObject {
  Q_OBJECT

  private:
   QStringList strlParams;
   QString m_localNum;
   
  public:
    MP10Parser();
    static unsigned int base64toint(const QString &strNum);
    static const QString inttobase64(unsigned int v, unsigned int count);
    void parseMessage(void);
    void cleanup(void);
    
  public slots:
    void initAuth(void);
    void splitLine(const QString &strBuffer);
    void handleServerMessage(const QStringList &strlArgs);
    void handleServerQuitMessage(const QStringList &strlArgs);
    void handleEndOfBurstMessage(const QStringList &strlArgs);
    void handleEndOfBurstAckMessage(const QStringList &strlArgs);
    void handlePingMessage(const QStringList &strlArgs);
    void handleNickMessage(const QStringList &strlArgs);
    void handleBurstMessage(const QStringList &strlArgs);
    void handleClearmodeMessage(const QStringList &strlArgs);
    void handleCreateMessage(const QStringList &strlArgs);
    void handleGlineMessage(const QStringList &strlArgs);
    void handleJoinMessage(const QStringList &strlArgs);
    void handleKickMessage(const QStringList &strlArgs);
    void handleKillMessage(const QStringList &strlArgs);
    void handleModeMessage(const QStringList &strlArgs);
    void handlePartMessage(const QStringList &strlArgs);
    void handlePrivmsgMessage(const QStringList &strlArgs);
    void handleQuitMessage(const QStringList &strlArgs);
    void handleTopicMessage(const QStringList &strlArgs);
    
  signals:
    void handleMessage(const QString &);
    void onServerMessage(const QStringList &);
    void onServerQuitMessage(const QStringList &);
    void onEndOfBurstMessage(const QStringList &);
    void onEndOfBurstAckMessage(const QStringList &);
    void onPingMessage(const QStringList &);
    void onNickMessage(const QStringList &);
    void onBurstMessage(const QStringList &);
    void onClearmodeMessage(const QStringList &);
    void onCreateMessage(const QStringList &);
    void onGlineMessage(const QStringList &);
    void onJoinMessage(const QStringList &);
    void onKickMessage(const QStringList &);
    void onKillMessage(const QStringList &);
    void onModeMessage(const QStringList &);
    void onPartMessage(const QStringList &);
    void onPrivmsgMessage(const QStringList &);
    void onQuitMessage(const QStringList &);
    void onTopicMessage(const QStringList &);
};

static const char convert2y[] = {
  'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P',
  'Q','R','S','T','U','V','W','X','Y','Z','a','b','c','d','e','f',
  'g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v',
  'w','x','y','z','0','1','2','3','4','5','6','7','8','9','[',']'
};

static const quint32 convert2n[] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   52,53,54,55,56,57,58,59,60,61, 0, 0, 0, 0, 0, 0,
   0, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
  15,16,17,18,19,20,21,22,23,24,25,62, 0,63, 0, 0,
   0,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
  41,42,43,44,45,46,47,48,49,50,51, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

#endif /* __MS_P10_PARSER_H */
