/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <dlfcn.h>
#include <MS_Dl.h>
#include <MS_Main.h>
#include <MS_Virtusers.h>

QList<MModule *> modList;
QString globalDlError;

MModule::MModule(const QString &strFilePath) {
  m_filePath.setFile(strFilePath);
  modList.append(this);
  
  globalDlError = QString("");
  m_loadTime = QDateTime::currentDateTime();
  m_pluginLoader = new QPluginLoader(strFilePath);
  QObject *mod = m_pluginLoader->instance();

  if (mod) {
    m_interface = qobject_cast<MModInterface *>(mod);
    if (m_interface) {
      globalMain->log->print(L_NOTICE, QString("Module \"%1\" (%2) loaded successfully.").arg(m_name).arg(m_version));
      return;
    }
  }
  
  globalDlError = m_pluginLoader->errorString();
  globalMain->log->print(L_ERR, QString("Loading module %1 failed: %2").arg(strFilePath).arg(globalDlError));
  delete this;
}

MModule::~MModule() {
  foreach(MVirtualUser *vuser, vuserList) {
    if (vuser->module() == this) {
      delete vuser;
    }
  }
  
  delete m_pluginLoader->instance();
  m_pluginLoader->unload();
  modList.removeOne(this);
}

void MModule::setName(const QString &strName) {
  m_name = strName;
}

void MModule::setVersion(const QString &strVer) {
  m_version = strVer;
}

void MModule::setBuilt(const QString &strBuilt) {
  m_built = strBuilt;
}

void MModule::setSysMod(const bool &is_sysmod) {
  m_sysMod = is_sysmod;
}

const QString &MModule::name(void) {
  return m_name;
}

const QString &MModule::version(void) {
  return m_version;
}

const QString &MModule::built(void) {
  return m_built;
}

const bool &MModule::isSysMod(void) {
  return m_sysMod;
}

const QFileInfo &MModule::filePath(void) {
  return m_filePath;
}

const QDateTime &MModule::loadTime(void) {
  return m_loadTime;
}
