/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>
#include <cstdlib>
#include <QApplication>
#include <QStringList>
#include <MS_Main.h>
#include <MS_Dl.h>

/* global main instance */
MMainInstance *globalMain;

MMainInstance::MMainInstance() {
  setBootTime(QDateTime::currentDateTime());
  m_application = 0;
  config = 0;
  socket = 0;
  log = 0;
  qsrand(QDateTime::currentDateTime().toTime_t());
}

void MMainInstance::setApplication(QApplication *psrc) {
  m_application = psrc;
}

QApplication *MMainInstance::getApplication(void) {
  return m_application;
}

void MMainInstance::setConfigFile(const QString &strFile) {
  m_strConfigFile = strFile;
}

const QString &MMainInstance::getConfigFile(void) {
  return m_strConfigFile;
}

void MMainInstance::setLogFile(const QString &strFile) {
  m_strLogFile = strFile;
}

const QString &MMainInstance::getLogFile(void) {
  return m_strLogFile;
}

void MMainInstance::parseArguments(void) {
  QStringList strlArguments;
  int i = 1;
  
  if (!m_application)
    return; /* application pointer not set yet */
  
  for(strlArguments = m_application->arguments(); i < strlArguments.size(); i++) {
    QString strCurrent = strlArguments.at(i);
    
    if (strCurrent == QString("--help") || strCurrent == QString("-h"))
      printUsage(strlArguments.at(0));
    else if ((strCurrent == QString("--config-file") || strCurrent == QString("-f")) && (i + 1) < strlArguments.size())
      setConfigFile(strlArguments.at(++i));
    else if ((strCurrent == QString("--log-file") || strCurrent == QString("-L")) && (i + 1) < strlArguments.size())
      setLogFile(strlArguments.at(++i));
    else if (strCurrent == QString("--nolog"))
      setLogFile(QString(""));
    else if (strCurrent == QString("--version") || strCurrent == QString("-v"))
      printVersion();
    else
      std::cout << "Unknown option or missing parameters: " << strCurrent.toStdString() << std::endl;
  }
}

void MMainInstance::printUsage(const QString &strExec) {
  std::cout << "Usage: " << strExec.toStdString() << " [options]" << std::endl;
  std::cout << "You may use one of the following options:" << std::endl;
  std::cout << "   --config-file, -f FILE   Path to the config file." << std::endl;
  std::cout << "                            [default: " << MSTR_DEFAULTCONFIG << "]" << std::endl;
  std::cout << "   --help, -h               Displays the command line usage list." << std::endl;
  std::cout << "   --log-file, -L FILE      Path to the log file." << std::endl;
  std::cout << "                            [default: " << MSTR_LOGFILE << "]" << std::endl;
  std::cout << "   --nolog                  Don't log to file." << std::endl;
  std::cout << "   --version, -v            Displays the program version." << std::endl;
  
  std::exit(0);
}

void MMainInstance::printVersion(void) {
  std::cout << QString("%1, Version %2").arg(m_application->applicationName()).arg(m_application->applicationVersion())
                                        .toStdString() << std::endl;
  std::cout << "Built " << __DATE__ << " " << __TIME__ << std::endl;
  std::cout << "Copyright(C) 2007, 2008 Christian Jurk <commx@commx.ws>" << std::endl;
  std::cout << "This program is released under the terms of the GNU Public License." << std::endl;
  
  std::exit(0);
}

void MMainInstance::setBootTime(const QDateTime &timestamp) {
  m_bootTime = timestamp;
}

const QDateTime &MMainInstance::getBootTime(void) {
  return m_bootTime;
}

QString MMainInstance::randomKey(const quint16 &length) {
  QString strChars = QString("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789");
  QString result;
  
  for(quint16 i = 0; i < length; i++) {
    result.append(strChars.at(qrand() % strChars.length()));
  }
  
  return result;
}

void MMainInstance::init(void) {
  std::cout << QString("%1, Version %2 initializing.").arg(m_application->applicationName())
                                        .arg(m_application->applicationVersion()).toStdString() << std::endl;
  m_builtTime = QString("%1 %2").arg(__DATE__).arg(__TIME__);
  log = new MLog(getLogFile());
  config = new MConfigReader(getConfigFile());
  socket = new MTcpSocket(config->get("server.host", "127.0.0.1"), config->get("server.port", "4400").toUShort());
#ifdef USE_P10
  parser = new MP10Parser();
#endif

  /* load the modules */
  QStringList modList = config->getModuleList();
  foreach(QString strmod, modList)
    new MModule(strmod);
}

const QString &MMainInstance::getBuilt(void) {
  return m_builtTime;
}
