/**
 * ModServ IRC Services
 * Version 2009
 * Copyright (C) 2007, 2008 Christian Jurk <commx@commx.ws>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License,
 * or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <MS_Servers.h>
#include <MS_Main.h>

QHash<unsigned int, MServer *> serverList;
MServer *uplinkServer = 0; /* pointer to the uplink server */
MServer *virtualServer = 0; /* pointer to our virtual services server */

MServer::MServer() {
  /* nothing */
}

void MServer::setServerName(const QString &strName) {
  m_serverName = strName;
}

void MServer::setServerDescription(const QString &strDesc) {
  m_serverDescription = strDesc;
}

void MServer::setServerNumeric(unsigned int num) {
  m_numeric = num;
}

void MServer::setServerFlags(unsigned short flags) {
  m_flags = flags;
}

void MServer::setServerState(char state) {
  m_state = state;
}

void MServer::setServerTSBooted(const QDateTime &timestamp) {
  m_timeBooted = timestamp;
}

void MServer::setServerTSLinked(const QDateTime &timestamp) {
  m_timeLinked = timestamp;
}

const QString &MServer::name(void) {
  return m_serverName;
}

const QString &MServer::description(void) {
  return m_serverDescription;
}

unsigned int MServer::numeric(void) {
  return m_numeric;
}

const QString MServer::numeric64(void) {
  QString strNumeric = globalMain->parser->inttobase64(m_numeric, 2);
  return strNumeric;
}

unsigned short MServer::flags(void) {
  return m_flags;
}

char MServer::state(void) {
  return m_state;
}

const QDateTime &MServer::bootTS(void) {
  return m_timeBooted;
}

const QDateTime &MServer::linkTS(void) {
  return m_timeLinked;
}

MServer *findServerByName(const QString &strName) {
  QHash<unsigned int, MServer *>::iterator si;
  
  /* note: need to use STL-style iterators as QHashIterator causes SIGSEGV */
  for(si = serverList.begin(); si != serverList.end(); ++si) {
    MServer *srv = si.value();
    QString cmpName = srv->name();
    
    if (strName.compare(cmpName, Qt::CaseInsensitive) == 0)
      return srv;
  }
  
  return 0;
}

MServer *findServer(const QString &strName) {
  unsigned int num = globalMain->parser->base64toint(strName);
  
  if (serverList.contains(num))
    return serverList.value(num);

  return 0;
}
